<?php
/**
 * An general object factory
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class ObjectFactory extends BaseFactory {
	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of this class
	 * @return	void
	 */
	protected function __construct ($className = __CLASS__) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Creates a new object given by the name or throws an exception if
	 * the class was not found. No parameters for the object are currently
	 * supported.
	 *
	 * @param	$className			Name of the class we shall construct
	 * @param	$args				Arguments in an indexed array
	 * @return	$objectInstance		An instance of the requested object
	 * @throws	NoClassException	If the requested class was not found
	 * @throws	EmptyVariableException	If a variable is empty unexpectly
	 */
	public static final function createObjectByName ($className, array $args=array()) {
		// First get an instance of this factory
		$factoryInstance = new ObjectFactory();

		// Is the class name valid and is the class there?
		if (empty($className)) {
			// Throw an exception here
			throw new EmptyVariableException(array($factoryInstance, 'className'), self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
		} elseif (!class_exists($className)) {
			// Then throw an exception
			throw new NoClassException(array($factoryInstance, $className), self::EXCEPTION_CLASS_NOT_FOUND);
		}

		// Create method name
		$methodName = sprintf('create%s', $className);

		// Run the user function
		$objectInstance = call_user_func_array(array($className, $methodName), $args);

		// Count this one up
		self::countObject($className);

		// Return the prepared instance
		return $objectInstance;
	}

	/**
	 * Creates an object by it's configured name
	 *
	 * @param	$configEnttry		Configuration entry to read
	 * @param	$args				Arguments in an indexed array
	 * @return	$objectInstance		An instance of the requested object
	 */
	public static final function createObjectByConfiguredName ($configEntry, array $args=array()) {
		// Read the configuration entry
		$className = FrameworkConfiguration::getSelfInstance()->getConfigEntry($configEntry);

		// Send this to the other factory...
		$objectInstance = self::createObjectByName($className, $args);

		// Return the instance
		return $objectInstance;
	}
}

// [EOF]
?>
